﻿--[[	Detailed Durability
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--	Note: PaperDollFrame uses GetInventoryItemTexture() to detect if there's an item in an inventory slot
--	For consistency, we do that too

----------------------------------
--[[	Constants & Helpers	]]
----------------------------------
--[[	WOW_PROJECT_ID contains either of the following constants
	WOW_PROJECT_MAINLINE = 1		Modern WoW
	WOW_PROJECT_CLASSIC = 2			Classic Vanilla
	WOW_PROJECT_BURNING_CRUSADE_CLASSIC =5	Classic Burning Crusade
--]]
local IsWoWClassic=(WOW_PROJECT_ID~=WOW_PROJECT_MAINLINE);

local SlotInventoryMap={--	AlertStatus-to-Inventory Slot Mapping
	INVSLOT_HEAD;
	INVSLOT_SHOULDER;
	INVSLOT_CHEST;
	INVSLOT_WAIST;
	INVSLOT_LEGS;
	INVSLOT_FEET;
	INVSLOT_WRIST;
	INVSLOT_HAND;
	INVSLOT_MAINHAND;
	INVSLOT_OFFHAND;
	INVSLOT_RANGED;
};

local AmmoCountMap; if IsWoWClassic then
	AmmoCountMap={--	Weapon Subtype-to-Inventory Slot Ammo Mapping
		[LE_ITEM_WEAPON_BOWS]		=INVSLOT_AMMO;
		[LE_ITEM_WEAPON_GUNS]		=INVSLOT_AMMO;
		[LE_ITEM_WEAPON_THROWN]		=INVSLOT_RANGED;
		[LE_ITEM_WEAPON_CROSSBOW]	=INVSLOT_AMMO;
	};
end

local function GetPercentColor(percent) return math.max(math.min(2*(1-percent),1),0),math.max(math.min(2*percent,1),0),0; end

--------------------------
--[[	Text Setup	]]
--------------------------
local DurabilityText=DurabilityFrame:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
DurabilityText:SetPoint("TOPRIGHT",DurabilityHead,"TOPLEFT");

local BrokenText=DurabilityFrame:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
BrokenText:SetPoint("BOTTOMLEFT",DurabilityText,"TOPLEFT");
BrokenText:SetTextColor(1,0,0);

local AmmoText; if IsWoWClassic then
	AmmoText=DurabilityFrame:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
	AmmoText:SetPoint("BOTTOMRIGHT",DurabilityRanged,"BOTTOMRIGHT");
end

--------------------------
--[[	Extra Events	]]
--------------------------
--	By default, DurabilityFrame only listens to PLAYER_ENTERING_WORLD and UPDATE_INVENTORY_ALERTS
--	The first fires on-load and the second only fires when GetInventoryAlertStatus() changes
DurabilityFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");--	Equipped/Unequipped/Swapped items
DurabilityFrame:RegisterEvent("UPDATE_INVENTORY_DURABILITY");--	Item took durability damage
if IsWoWClassic then
	DurabilityFrame:RegisterEvent("BAG_UPDATE_DELAYED");--	Ammo count
end

------------------
--[[	Hooks	]]
------------------
--	Called to index INVENTORY_ALERT_COLORS with its return value (0=Normal (no color); 1=Low Durability; 2=Broken)
local AlertStatusIndex;
hooksecurefunc("GetInventoryAlertStatus",function(index) AlertStatusIndex=index; end);

--	Hook INVENTORY_ALERT_COLORS to hand over our custom color
local CustomColor=CreateColor(1,1,1);--	Init Color Object (It's just an RGB table with metamethods added)
INVENTORY_ALERT_COLORS=setmetatable({},{__index=function()
	local invslot=SlotInventoryMap[AlertStatusIndex];--	AlertStatusIndex is the index given to GetInventoryAlertStatus()
	if GetInventoryItemTexture("player",invslot) then--	Detect item in inventory slot
		local cur,max=GetInventoryItemDurability(invslot);
		if cur and max then--	Item has durability?
			local percent=max>0 and cur/max or 1;
			CustomColor:SetRGB(GetPercentColor(percent));--	Fades between green, yellow, and red
		else CustomColor:SetRGB(0,0,1); end--	Item is indestructible
	else return nil; end--	Hide

	return CustomColor;
end});

----------------------------------
--[[	Text Update Hook	]]
----------------------------------
local function UpdateDurability()
--	Scan Items
	local numbroken,numitems=0,0;
	local lowpercent;
	for _,slot in pairs(SlotInventoryMap) do
		if GetInventoryItemTexture("player",slot) then--	Detect item in inventory slot
			local cur,max=GetInventoryItemDurability(slot);
			if cur and max then--	Process if item is destructible
				if cur>0 then lowpercent=math.min(cur/max,lowpercent or 1);--	Store lowest percent
				else numbroken=numbroken+1; end
				numitems=numitems+1;
			end
		end
	end

--	Durability Text
	lowpercent=lowpercent or 0;--	Default to zero if everything is broken
	if numitems>0 then--	We have durability to display
		DurabilityText:SetFormattedText("%.0f%%",lowpercent*100);
		DurabilityText:SetTextColor(GetPercentColor(lowpercent));
	else DurabilityText:SetText(nil); end--	Nothing to show here, move along

--	Broken Text
	if numbroken>0 then BrokenText:SetFormattedText("%d/%d Broken",numbroken,numitems);
	else BrokenText:SetText(nil); end
end

if IsWoWClassic then
	hooksecurefunc("DurabilityFrame_SetAlerts",function()
		UpdateDurability();--	Update Durability Display

--		Identify ammo type
		local itemid,invslot; do
			local typeid,subid;
			itemid=GetInventoryItemID("player",INVSLOT_RANGED);
			if itemid then--	Do we have an item in the ranged slot?
				local _; _,_,_,_,_,typeid,subid=GetItemInfoInstant(itemid);
				if typeid==LE_ITEM_CLASS_WEAPON then--	Is it a weapon?
					invslot=AmmoCountMap[subid];--	Lookup the ammo slot based on subtype
					itemid=GetInventoryItemID("player",invslot);--	Update ItemID
				end
				if itemid then GetItemInfo(itemid); end--	Cache item
			end
		end

--		Display ammo count
		if invslot then--	Detect item in inventory slot
--			Stack Size & Count (Stack defaults to 1; Count defaults to 0; Avoids division by zero when generating color)
			local stack=itemid and select(8,GetItemInfo(itemid)) or 1;
			local count=itemid and GetInventoryItemCount("player",invslot) or 0;--	Gets counts from inventory slot (thrown items use stack in slot while ammo points to bags)

--			Formatting (Stacks+Remain or Count where appropriate)
			if count>stack then AmmoText:SetFormattedText("%d+%d",math.floor(count/stack),count%stack);
			else AmmoText:SetText(count); end
			AmmoText:SetTextColor(GetPercentColor(count/stack));
		else AmmoText:SetText(nil); end
	end);
else hooksecurefunc(DurabilityFrame,"SetAlerts",UpdateDurability); end
